/**
 * mePed_TestServoRange, v1.0, 1/10/2018
 * 
 * Provides testing and experimentation on all mePed servos individually.  User can select
 * any one servo, then position that servo to any position (within limits of mePed mechanics)
 * to check performance and ranges of each servo, to check for correct assembly and failures.
 * 
 * mePed robot body must be suspended so each servo can move freely throughout the entire 
 * range.  Set the body up on a coffee can, food storage container, small box, etc.
 * 
 * To run this application, the Arduino IDE is used for the Monitor display and command input.
 * (Tools > Serial Monitor).  See the Arduino site (https://www.arduino.cc/en/Main/Software).
 * You also need the IDE to load this application to the mePed, replacing the original
 * starter program.  Connect your PC to the mePed via a USB cable, upload this application to
 * the mePed, and open the Serial Monitor from the IDE.  From the Monitor's command line,
 * enter servo selection, angle or microsecond value for positioning, etc. as described by
 * the help display on the monitor panel.
 * 
 * After testing/experimenting with this application, the original mePed application is 
 * available for re-loading from:
 * http://www.meped.io/sites/default/files/2017-06/mePed_IR_Starter_Program_0.ino).

 * 
 * @author Mark Henrichs
 */

#include <Servo.h>

//===== Globals ============================================================================
String  inputStr;   // Input string entered by user

// Define 8 Servos
Servo   myServo[9]; // Need 8 servos, but myServo[0] is not used
/*
  Servo myServo[1] = Front Left Pivot Servo
  Servo myServo[2] = Front Left Lift Servo
  Servo myServo[3] = Back Left Pivot Servo
  Servo myServo[4] = Back Left Lift Servo
  Servo myServo[5] = Back Right Pivot Servo
  Servo myServo[6] = Back Right Lift Servo
  Servo myServo[7] = Front Right Pivot Servo
  Servo myServo[8] = Front Right Lift Servo
*/
Servo* selServo = &myServo[1];  //Currently selected servo pointer
int   selServoNum = 1;

//===== Setup ==============================================================================

void setup() {

  // Attach servos to Arduino Pins
  myServo[1].attach(2);
  myServo[2].attach(3);
  myServo[3].attach(4);
  myServo[4].attach(5);
  myServo[5].attach(6);
  myServo[6].attach(7);
  myServo[7].attach(8);
  myServo[8].attach(9);
  //myServo[0] is not used, just for human readability of code, and consistency with mePed code

  Serial.begin(9600);
  Serial.println("mePed Servo Range Test");
  showHelp();
  showServoStat();
}

//===== Main Loop ==============================================================================

void loop() {

  while (Serial.available()) {
    inputStr = Serial.readString();   //Read user input from serial monitor (Tools > Serial Monitor)
  }

  if (inputStr.length() > 0) {
    inputStr.trim();                  //Strip any spaces front and back
    Serial.print("Received input '" + inputStr + "', ");  //Display user input string

    char firstChar = inputStr.charAt(0);
    if ( isDigit(firstChar) ) {
      //Must be a numeric value, either angle or microseconds
      int n = inputStr.toInt();   //Convert inputStr to a number

      //Based on value input, use either as microseconds (650-2350) or angle (10-180) in degrees
      //Microsecond range is actually 500-2500, but is limited here to help protect mePed servo ranges
      if ( (n >= 650) && (n <= 2450) ) {
        Serial.print("writing Microseconds to servo: ");
        Serial.println(n);
        selServo->writeMicroseconds(n);
      } else if ( (n >= 10) && (n <= 180) ) {
        Serial.print("writing Angle to servo: ");
        Serial.println(n);
        selServo->write(n);
      } else {
        Serial.println("NUMERIC INPUT VALUE ERROR!");
        showHelp();
      }
      //End of if Numeric value
    } else {
      //Must be a control character, or 's' for servo number following
      char secondChar = '_';    //Set invalid char as flag if user does not enter 2nd char
      if (inputStr.length() > 1) {
        secondChar = inputStr.charAt(1);
      }
      if ( (firstChar == 's') || (firstChar == 'S') ) {
        int servoNum = secondChar - '0';    //Convert char to int
        if ( (servoNum >= 1) && (servoNum <= 8) ) {
          selServo = &myServo[servoNum];    //Currently selected servo
          selServoNum = servoNum;
          Serial.println("selecting servo " + inputStr);
        } else {
          Serial.println("SERVO NUMBER OUT OF RANGE (1-8)!");
          showHelp();
        }
      } else if ( (firstChar == 'h') || (firstChar == 'H') ) {
        Serial.println("printing Help");
        showHelp();
      } else if ( (firstChar == 'c') || (firstChar == 'C') ) {
        if ( (secondChar == 'a') || (secondChar == 'A') ) {
          Serial.println("centering ALL servos");
          centerServos();
        } else {
          Serial.println("centering current servo");
          selServo->write(90);
          delay(200);
        }
      } else if ( (firstChar == 'a') || (firstChar == 'A') ) {
        Serial.print("servo angles: ");
        for (int n = 1; n <= 8; n++) {
          String angleStr = "s";
          angleStr.concat(n);
          angleStr.concat("=");
          angleStr.concat(myServo[n].read());
          angleStr.concat(" ");
          Serial.print(angleStr);
        }
        Serial.println();
      } else {
        Serial.println("ALPABETIC INPUT VALUE ERROR!");
        showHelp();
      }
      //End of else String value
    } 
    showServoStat();
    inputStr = "";  //Clear input string to wait for new user value

  }
}

//===== Show Help =============================================================================

void showHelp() {
  Serial.println("");
  Serial.println("Enter a number 10-180 to test angle in degrees.  Enter a number 650-2350 to specify Microseconds.");
  Serial.println("Enter 's1' - 's8' to select a servo (1-8) for subsequent commands.  Servos control the following:");
  Serial.println("   s1 = Front Left Pivot Servo");
  Serial.println("   s2 = Front Left Lift Servo");
  Serial.println("   s3 = Back Left Pivot Servo");
  Serial.println("   s4 = Back Left Lift Servo");
  Serial.println("   s5 = Back Right Pivot Servo");
  Serial.println("   s6 = Back Right Lift Servo");
  Serial.println("   s7 = Front Right Pivot Servo");
  Serial.println("   s8 = Front Right Lift Servo");
  Serial.println("Enter 'c' to center the current servo, or 'ca' to center all servos.");
  Serial.println("Enter 'a' to display angles of each servo.  Enter 'h' to re-display this help.");
  Serial.println("");
  Serial.println("**CAUTION!**  Test range limits carefully to avoid servo damage!");
  Serial.println("");
}

//== Show current servo status ================================================================

void showServoStat() {
  Serial.print("Currently selected servo is s");
  Serial.print(selServoNum);
  Serial.print(", angle=");
  Serial.println(selServo->read());
}

//== Center Servos ============================================================================

void centerServos() {
  for (int x = 1; x < 9; x++) {
    myServo[x].write(90);
  }
  delay(200);   //Wait for completion
}




